package mx.messaging
{
   import flash.errors.IllegalOperationError;
   import flash.events.EventDispatcher;
   import flash.events.TimerEvent;
   import flash.utils.Timer;
   import mx.collections.ArrayCollection;
   import mx.core.mx_internal;
   import mx.events.PropertyChangeEvent;
   import mx.logging.ILogger;
   import mx.logging.Log;
   import mx.messaging.config.LoaderConfig;
   import mx.messaging.errors.InvalidDestinationError;
   import mx.messaging.events.ChannelEvent;
   import mx.messaging.events.ChannelFaultEvent;
   import mx.messaging.messages.AbstractMessage;
   import mx.messaging.messages.CommandMessage;
   import mx.messaging.messages.IMessage;
   import mx.utils.Translator;
   import mx.utils.URLUtil;
   
   use namespace mx_internal;
   
   public class Channel extends EventDispatcher
   {
      private static const dep:ArrayCollection = null;
      
      private static const TRANSLATOR:Translator = Translator.getDefaultInstanceFor(Channel);
      
      protected var _log:ILogger;
      
      private var _failoverIndex:int;
      
      private var _connected:Boolean = false;
      
      private var _reconnecting:Boolean = false;
      
      private var _primaryURI:String;
      
      private var _channelSets:Array;
      
      private var _connectTimeout:int = -1;
      
      private var _id:String;
      
      private var _uri:String;
      
      mx_internal var logoutAllowed:Boolean;
      
      mx_internal var loggedIn:Boolean;
      
      private var _connecting:Boolean;
      
      private var _connectTimer:Timer;
      
      private var _shouldBeConnected:Boolean;
      
      private var _requestTimeout:int = -1;
      
      private var _endpoint:String;
      
      protected var credentials:String;
      
      private var _failoverURIs:Array;
      
      public function Channel(param1:String, param2:String)
      {
         _channelSets = [];
         _connected = false;
         _connectTimeout = -1;
         _reconnecting = false;
         _requestTimeout = -1;
         super();
         _log = Log.getLogger("mx.messaging.Channel");
         _connecting = false;
         _failoverIndex = -1;
         _id = param1;
         _primaryURI = param2;
         _shouldBeConnected = false;
         _uri = param2;
         calculateEndpoint();
         mx_internal::logoutAllowed = true;
      }
      
      protected function disconnectFailed(param1:ChannelFaultEvent) : void
      {
         _connecting = false;
         setConnected(false);
         if(Log.isError())
         {
            _log.error("\'{0}\' channel disconnect failed.",id);
         }
         if(reconnecting)
         {
            resetToPrimaryURI();
            param1.reconnecting = false;
         }
         dispatchEvent(param1);
      }
      
      public function get uri() : String
      {
         return _uri;
      }
      
      [Bindable(event="propertyChange")]
      public function get connected() : Boolean
      {
         return _connected;
      }
      
      private function shutdownConnectTimer() : void
      {
         if(_connectTimer != null)
         {
            _connectTimer.stop();
            _connectTimer = null;
         }
      }
      
      public function get endpoint() : String
      {
         return _endpoint;
      }
      
      public function get connectTimeout() : int
      {
         return _connectTimeout;
      }
      
      public function get protocol() : String
      {
         throw new IllegalOperationError("Channel subclasses must override " + "the get function for \'protocol\' to return the proper protocol " + "string.");
      }
      
      private function initializeRequestTimeout(param1:MessageResponder) : void
      {
         var _loc2_:IMessage = null;
         _loc2_ = param1.message;
         if(_loc2_.headers[AbstractMessage.REQUEST_TIMEOUT_HEADER] != null)
         {
            param1.startRequestTimeout(_loc2_.headers[AbstractMessage.REQUEST_TIMEOUT_HEADER]);
         }
         else if(requestTimeout > 0)
         {
            param1.startRequestTimeout(requestTimeout);
         }
      }
      
      final public function disconnect(param1:ChannelSet) : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         _loc2_ = int(_channelSets.length);
         _loc3_ = 0;
         while(_loc3_ < _loc2_)
         {
            if(_channelSets[_loc3_] == param1)
            {
               _channelSets.splice(_loc3_,1);
               removeEventListener(ChannelEvent.CONNECT,param1.channelConnectHandler,false);
               removeEventListener(ChannelEvent.DISCONNECT,param1.channelDisconnectHandler,false);
               removeEventListener(ChannelFaultEvent.FAULT,param1.channelFaultHandler,false);
               if(connected)
               {
                  param1.channelDisconnectHandler(ChannelEvent.createEvent(ChannelEvent.DISCONNECT,this,false));
               }
               break;
            }
            _loc3_++;
         }
         if(_channelSets.length == 0)
         {
            _shouldBeConnected = false;
            if(connected)
            {
               internalDisconnect();
            }
         }
      }
      
      public function get failoverURIs() : Array
      {
         return _failoverURIs != null ? _failoverURIs : [];
      }
      
      public function get id() : String
      {
         return _id;
      }
      
      public function set requestTimeout(param1:int) : void
      {
         _requestTimeout = param1;
      }
      
      final public function send(param1:MessageAgent, param2:IMessage) : void
      {
         var _loc3_:MessageResponder = null;
         if(param2.destination.length == 0)
         {
            if(param1.destination.length == 0)
            {
               throw new InvalidDestinationError(TRANSLATOR.textOf("noDestinationSpecified"));
            }
            param2.destination = param1.destination;
         }
         if(Log.isDebug())
         {
            _log.debug("\'{0}\' channel sending message:\n{1}",id,param2.toString());
         }
         param2.headers[AbstractMessage.ENDPOINT_HEADER] = id;
         _loc3_ = getMessageResponder(param1,param2);
         initializeRequestTimeout(_loc3_);
         internalSend(_loc3_);
      }
      
      [Bindable(event="propertyChange")]
      public function get reconnecting() : Boolean
      {
         return _reconnecting;
      }
      
      public function logout(param1:MessageAgent) : void
      {
         var _loc2_:CommandMessage = null;
         this.credentials = null;
         if(connected && mx_internal::logoutAllowed)
         {
            _loc2_ = new CommandMessage();
            _loc2_.operation = CommandMessage.LOGOUT_OPERATION;
            _loc2_.messageRefType = CommandMessage.AUTHENTICATION_MESSAGE_REF_TYPE;
            internalSend(new AuthenticationMessageResponder(param1,_loc2_,this,_log));
            mx_internal::logoutAllowed = false;
         }
      }
      
      private function setReconnecting(param1:Boolean) : void
      {
         var _loc2_:PropertyChangeEvent = null;
         if(_reconnecting != param1)
         {
            _loc2_ = PropertyChangeEvent.createUpdateEvent(this,"reconnecting",_reconnecting,param1);
            _reconnecting = param1;
            dispatchEvent(_loc2_);
         }
      }
      
      public function applySettings(param1:XML) : void
      {
         var _loc2_:XML = null;
         if(Log.isInfo())
         {
            _log.info("\'{0}\' channel settings are:\n{1}",id,param1);
         }
         if(param1.properties.length())
         {
            _loc2_ = param1.properties[0];
            if(_loc2_["connect-timeout-seconds"].length())
            {
               connectTimeout = _loc2_["connect-timeout-seconds"].toString();
            }
         }
      }
      
      public function set failoverURIs(param1:Array) : void
      {
         if(param1 != null)
         {
            _failoverURIs = param1;
            _failoverIndex = -1;
         }
      }
      
      protected function getMessageResponder(param1:MessageAgent, param2:IMessage) : MessageResponder
      {
         throw new IllegalOperationError("Channel subclasses must override " + " getMessageResponder().");
      }
      
      protected function connectSuccess() : void
      {
         shutdownConnectTimer();
         _connecting = false;
         setConnected(true);
         _failoverIndex = -1;
         if(Log.isInfo())
         {
            _log.info("\'{0}\' channel is connected.",id);
         }
         dispatchEvent(ChannelEvent.createEvent(ChannelEvent.CONNECT,this,reconnecting));
         setReconnecting(false);
      }
      
      private function shouldAttemptFailover() : Boolean
      {
         return _shouldBeConnected && _failoverURIs != null && _failoverURIs.length > 0;
      }
      
      public function set connectTimeout(param1:int) : void
      {
         _connectTimeout = param1;
      }
      
      public function setCredentials(param1:String, param2:MessageAgent = null) : void
      {
         var _loc3_:CommandMessage = null;
         this.credentials = param1;
         if(connected && param1 != null)
         {
            _loc3_ = new CommandMessage();
            _loc3_.operation = CommandMessage.LOGIN_OPERATION;
            _loc3_.messageRefType = CommandMessage.AUTHENTICATION_MESSAGE_REF_TYPE;
            _loc3_.body = param1;
            internalSend(new AuthenticationMessageResponder(param2,_loc3_,this,_log));
         }
      }
      
      protected function connectTimeoutHandler(param1:TimerEvent) : void
      {
         var _loc2_:ChannelFaultEvent = null;
         shutdownConnectTimer();
         if(!connected)
         {
            _shouldBeConnected = false;
            _loc2_ = ChannelFaultEvent.createEvent(this,false,"Channel.Connect.Failed","error",TRANSLATOR.textOf("connectTimedOut"));
            connectFailed(_loc2_);
         }
      }
      
      protected function internalDisconnect(param1:Boolean = false) : void
      {
      }
      
      protected function setConnected(param1:Boolean) : void
      {
         var _loc2_:PropertyChangeEvent = null;
         if(_connected != param1)
         {
            _loc2_ = PropertyChangeEvent.createUpdateEvent(this,"connected",_connected,param1);
            _connected = param1;
            dispatchEvent(_loc2_);
         }
      }
      
      public function get requestTimeout() : int
      {
         return _requestTimeout;
      }
      
      mx_internal function sendClusterRequest(param1:MessageResponder) : void
      {
         internalSend(param1);
      }
      
      private function reconnect(param1:TimerEvent) : void
      {
         internalConnect();
      }
      
      protected function connectFailed(param1:ChannelFaultEvent) : void
      {
         setConnected(false);
         if(Log.isError())
         {
            _log.error("\'{0}\' channel connect failed.",id);
         }
         if(shouldAttemptFailover())
         {
            _connecting = true;
            failover();
         }
         else
         {
            _connecting = false;
         }
         if(reconnecting)
         {
            param1.reconnecting = true;
         }
         dispatchEvent(param1);
      }
      
      private function resetToPrimaryURI() : void
      {
         _connecting = false;
         setReconnecting(false);
         _uri = _primaryURI;
         calculateEndpoint();
         _failoverIndex = -1;
      }
      
      protected function internalSend(param1:MessageResponder) : void
      {
      }
      
      protected function disconnectSuccess(param1:Boolean = false) : void
      {
         setConnected(false);
         if(Log.isInfo())
         {
            _log.info("\'{0}\' channel disconnected.",id);
         }
         if(!param1 && shouldAttemptFailover())
         {
            _connecting = true;
            failover();
         }
         else
         {
            _connecting = false;
         }
         dispatchEvent(ChannelEvent.createEvent(ChannelEvent.DISCONNECT,this,reconnecting,param1));
      }
      
      protected function internalConnect() : void
      {
      }
      
      private function calculateEndpoint() : void
      {
         var _loc1_:String = null;
         var _loc2_:String = null;
         _loc1_ = _uri;
         _loc2_ = URLUtil.getProtocol(_loc1_);
         if(_loc2_.length == 0)
         {
            _loc1_ = URLUtil.getFullURL(LoaderConfig.url,_loc1_);
         }
         _loc1_ = URLUtil.replaceTokens(_loc1_);
         _loc2_ = URLUtil.getProtocol(_loc1_);
         if(_loc2_.length > 0)
         {
            _endpoint = URLUtil.replaceProtocol(_loc1_,protocol);
         }
         else
         {
            _endpoint = protocol + ":" + _loc1_;
         }
         if(Log.isInfo())
         {
            _log.info("\'{0}\' channel endpoint set to {1}",id,_endpoint);
         }
      }
      
      private function failover() : void
      {
         var _loc1_:Timer = null;
         ++_failoverIndex;
         if(_failoverIndex + 1 <= failoverURIs.length)
         {
            setReconnecting(true);
            _uri = failoverURIs[_failoverIndex];
            calculateEndpoint();
            if(Log.isInfo())
            {
               _log.info("\'{0}\' channel attempting to connect to {1}.",id,endpoint);
            }
            _loc1_ = new Timer(1,1);
            _loc1_.addEventListener(TimerEvent.TIMER,reconnect);
            _loc1_.start();
         }
         else
         {
            if(Log.isInfo())
            {
               _log.info("\'{0}\' channel has exhausted failover options and has reset to its primary endpoint.",id);
            }
            resetToPrimaryURI();
         }
      }
      
      public function get channelSets() : Array
      {
         return _channelSets;
      }
      
      final public function connect(param1:ChannelSet) : void
      {
         var _loc2_:Boolean = false;
         var _loc3_:int = 0;
         var _loc4_:int = 0;
         _loc2_ = false;
         _loc3_ = int(_channelSets.length);
         _loc4_ = 0;
         while(_loc4_ < _channelSets.length)
         {
            if(_channelSets[_loc4_] == param1)
            {
               _loc2_ = true;
               break;
            }
            _loc4_++;
         }
         _shouldBeConnected = true;
         if(!_loc2_)
         {
            _channelSets.push(param1);
            addEventListener(ChannelEvent.CONNECT,param1.channelConnectHandler);
            addEventListener(ChannelEvent.DISCONNECT,param1.channelDisconnectHandler);
            addEventListener(ChannelFaultEvent.FAULT,param1.channelFaultHandler);
         }
         if(connected)
         {
            param1.channelConnectHandler(ChannelEvent.createEvent(ChannelEvent.CONNECT,this,false));
         }
         else if(!_connecting)
         {
            _connecting = true;
            if(connectTimeout > 0)
            {
               _connectTimer = new Timer(connectTimeout * 1000,1);
               _connectTimer.addEventListener(TimerEvent.TIMER,connectTimeoutHandler);
               _connectTimer.start();
            }
            internalConnect();
         }
      }
   }
}

import mx.core.mx_internal;
import mx.logging.ILogger;
import mx.logging.Log;
import mx.messaging.events.ChannelFaultEvent;
import mx.messaging.messages.CommandMessage;
import mx.messaging.messages.ErrorMessage;
import mx.messaging.messages.IMessage;

use namespace mx_internal;

class AuthenticationMessageResponder extends MessageResponder
{
   private var _log:ILogger;
   
   public function AuthenticationMessageResponder(param1:MessageAgent, param2:IMessage, param3:Channel, param4:ILogger)
   {
      super(param1,param2,param3);
      _log = param4;
   }
   
   override protected function resultHandler(param1:IMessage) : void
   {
      var _loc2_:CommandMessage = null;
      _loc2_ = message as CommandMessage;
      if(_loc2_.operation == CommandMessage.LOGIN_OPERATION)
      {
         channel.mx_internal::loggedIn = true;
         channel.mx_internal::logoutAllowed = true;
         if(Log.isDebug())
         {
            _log.debug("Login successful");
         }
      }
      else
      {
         channel.mx_internal::loggedIn = false;
         if(Log.isDebug())
         {
            _log.debug("Logout successful");
         }
      }
   }
   
   override protected function statusHandler(param1:IMessage) : void
   {
      var _loc2_:CommandMessage = null;
      var _loc3_:ErrorMessage = null;
      var _loc4_:ChannelFaultEvent = null;
      _loc2_ = CommandMessage(message);
      if(_loc2_.operation == CommandMessage.LOGIN_OPERATION)
      {
         if(Log.isDebug())
         {
            _log.debug("Login failure: {0}",param1.toString());
         }
      }
      else
      {
         if(Log.isDebug())
         {
            _log.debug("Logout failure: {0}",param1.toString());
         }
         channel.mx_internal::logoutAllowed = true;
      }
      if(agent != null)
      {
         agent.fault(ErrorMessage(param1),message);
      }
      else
      {
         _loc3_ = ErrorMessage(param1);
         _loc4_ = ChannelFaultEvent.createEvent(channel,false,"Channel.Authentication.Error","warn",_loc3_.faultString);
         _loc4_.rootCause = _loc3_;
         channel.dispatchEvent(_loc4_);
      }
   }
}
