package mx.messaging
{
   import flash.events.TimerEvent;
   import flash.utils.Timer;
   import mx.core.mx_internal;
   import mx.events.PropertyChangeEvent;
   import mx.logging.Log;
   import mx.messaging.config.ServerConfig;
   import mx.messaging.events.ChannelEvent;
   import mx.messaging.events.ChannelFaultEvent;
   import mx.messaging.events.MessageEvent;
   import mx.messaging.messages.AbstractMessage;
   import mx.messaging.messages.AcknowledgeMessage;
   import mx.messaging.messages.CommandMessage;
   import mx.messaging.messages.ErrorMessage;
   import mx.messaging.messages.IMessage;
   import mx.utils.Translator;
   
   use namespace mx_internal;
   
   public class Consumer extends MessageAgent
   {
      private static const TRANSLATOR:Translator = Translator.getDefaultInstanceFor(Consumer);
      
      private var _timestamp:Number;
      
      private var _resubscribeInterval:int;
      
      private var _resubscribeAttempts:int;
      
      private var _messageType:String;
      
      private var _currentAttempt:int;
      
      private var _receiveMsg:CommandMessage;
      
      private var _subscribeMsg:CommandMessage;
      
      private var _selector:String;
      
      private var _resubscribeTimer:Timer;
      
      private var _subscribed:Boolean;
      
      private var _shouldBeSubscribed:Boolean;
      
      public function Consumer(param1:String = "flex.messaging.messages.AsyncMessage")
      {
         super();
         _log = Log.getLogger("mx.messaging.Consumer");
         _agentType = "consumer";
         _messageType = param1;
         _resubscribeAttempts = 5;
         _resubscribeInterval = 5000;
         _selector = "";
         _shouldBeSubscribed = false;
         _subscribed = false;
         _timestamp = -1;
      }
      
      mx_internal function dispatchMessageList(param1:Array) : void
      {
         var _loc2_:int = 0;
         _loc2_ = 0;
         while(_loc2_ < param1.length)
         {
            messageHandler(MessageEvent.createEvent(destination,param1[_loc2_]));
            _loc2_++;
         }
      }
      
      protected function getUnsubscribeMessage() : CommandMessage
      {
         var _loc1_:CommandMessage = null;
         _loc1_ = new CommandMessage();
         _loc1_.operation = CommandMessage.UNSUBSCRIBE_OPERATION;
         _loc1_.messageRefType = messageType;
         _loc1_.clientId = clientId;
         _loc1_.destination = destination;
         return _loc1_;
      }
      
      protected function startResubscribeTimer() : void
      {
         if(_shouldBeSubscribed && _resubscribeTimer == null)
         {
            if(_resubscribeAttempts != 0 && _resubscribeInterval > 0)
            {
               if(Log.isDebug())
               {
                  _log.debug("\'{0}\' {1} starting resubscribe timer.",id,_agentType);
               }
               _receiveMsg = new CommandMessage();
               _receiveMsg.operation = CommandMessage.POLL_OPERATION;
               _receiveMsg.destination = destination;
               _receiveMsg.timestamp = timestamp;
               _receiveMsg.messageRefType = messageType;
               _resubscribeTimer = new Timer(1);
               _resubscribeTimer.addEventListener(TimerEvent.TIMER,resubscribe);
               _resubscribeTimer.start();
               _currentAttempt = 0;
            }
         }
      }
      
      private function setSubscribed(param1:Boolean) : void
      {
         var _loc2_:PropertyChangeEvent = null;
         if(_subscribed != param1)
         {
            _loc2_ = PropertyChangeEvent.createUpdateEvent(this,"subscribed",_subscribed,param1);
            _subscribed = param1;
            if(channelSet != null)
            {
               if(param1)
               {
                  channelSet.addEventListener(destination,messageHandler);
               }
               else
               {
                  channelSet.removeEventListener(destination,messageHandler);
               }
            }
            dispatchEvent(_loc2_);
         }
      }
      
      protected function resubscribe(param1:TimerEvent) : void
      {
         var _loc2_:ErrorMessage = null;
         if(_resubscribeAttempts != -1 && _currentAttempt >= _resubscribeAttempts)
         {
            stopResubscribeTimer();
            _shouldBeSubscribed = false;
            _loc2_ = new ErrorMessage();
            _loc2_.faultCode = "Client.Error.Subscribe";
            _loc2_.faultString = TRANSLATOR.textOf("consumerSubscribeError");
            _loc2_.faultDetail = TRANSLATOR.textOf("failedToSubscribe");
            _loc2_.correlationId = _subscribeMsg.messageId;
            fault(_loc2_,_subscribeMsg);
            return;
         }
         if(Log.isDebug())
         {
            _log.debug("\'{0}\' {1} trying to resubscribe.",id,_agentType);
         }
         _resubscribeTimer.delay = _resubscribeInterval;
         ++_currentAttempt;
         internalSend(_subscribeMsg);
         internalSend(_receiveMsg);
      }
      
      override public function acknowledge(param1:AcknowledgeMessage, param2:IMessage) : void
      {
         var _loc3_:CommandMessage = null;
         var _loc4_:Array = null;
         if(!param1.headers[AcknowledgeMessage.ERROR_HINT_HEADER] && param2 is CommandMessage)
         {
            _loc3_ = param2 as CommandMessage;
            switch(_loc3_.operation)
            {
               case CommandMessage.UNSUBSCRIBE_OPERATION:
                  super.mx_internal::setClientId(null);
                  setSubscribed(false);
                  if(Log.isInfo())
                  {
                     _log.info("\'{0}\' {1} acknowledge for unsubscribe.",id,_agentType);
                  }
                  break;
               case CommandMessage.SUBSCRIBE_OPERATION:
                  stopResubscribeTimer();
                  super.mx_internal::setClientId(param1.clientId);
                  if(param1.timestamp > _timestamp)
                  {
                     _timestamp = param1.timestamp - 1;
                  }
                  setSubscribed(true);
                  if(Log.isInfo())
                  {
                     _log.info("\'{0}\' {1} acknowledge for subscribe. New client id \'{2}\' new timestamp {3}",id,_agentType,param1.clientId,_timestamp);
                  }
                  break;
               case CommandMessage.POLL_OPERATION:
                  if(param1.body != null && param1.body is Array)
                  {
                     _loc4_ = param1.body as Array;
                     mx_internal::dispatchMessageList(_loc4_);
                  }
            }
         }
         super.acknowledge(param1,param2);
      }
      
      override protected function initChannelSet(param1:IMessage) : void
      {
         if(channelSet == null)
         {
            channelSet = ServerConfig.getChannelSet(destination,null,messageType);
            if(_credentials != null && channelSet != null)
            {
               channelSet.setCredentials(_credentials,this);
            }
         }
      }
      
      override public function fault(param1:ErrorMessage, param2:IMessage) : void
      {
         if(param1.headers[ErrorMessage.RETRYABLE_HINT_HEADER])
         {
            if(_resubscribeTimer == null)
            {
               if(_subscribeMsg != null && param1.correlationId == _subscribeMsg.messageId)
               {
                  _shouldBeSubscribed = false;
               }
               super.fault(param1,param2);
            }
         }
         else
         {
            super.fault(param1,param2);
         }
      }
      
      override public function disconnect() : void
      {
         _shouldBeSubscribed = false;
         if(channelSet != null)
         {
            channelSet.removeEventListener(destination,messageHandler);
         }
         stopResubscribeTimer();
         super.disconnect();
         setSubscribed(false);
      }
      
      override public function set destination(param1:String) : void
      {
         if(destination != param1)
         {
            if(subscribed)
            {
               unsubscribe();
               _shouldBeSubscribed = true;
            }
            super.destination = param1;
            if(_shouldBeSubscribed)
            {
               subscribe();
            }
         }
      }
      
      protected function stopResubscribeTimer() : void
      {
         if(_resubscribeTimer != null)
         {
            if(Log.isDebug())
            {
               _log.debug("\'{0}\' {1} stopping resubscribe timer.",id,_agentType);
            }
            _resubscribeTimer.removeEventListener(TimerEvent.TIMER,resubscribe);
            _resubscribeTimer.reset();
            _resubscribeTimer = null;
         }
      }
      
      public function set resubscribeInterval(param1:int) : void
      {
         if(_resubscribeInterval != param1)
         {
            if(param1 < 0)
            {
               throw new ArgumentError(TRANSLATOR.textOf("resubscribeIntervalNegative"));
            }
            if(param1 == 0)
            {
               stopResubscribeTimer();
            }
            else if(_resubscribeTimer != null)
            {
               _resubscribeTimer.delay = param1;
            }
            _resubscribeInterval = param1;
         }
      }
      
      public function set timestamp(param1:Number) : void
      {
         _timestamp = param1;
      }
      
      public function set resubscribeAttempts(param1:int) : void
      {
         if(_resubscribeAttempts != param1)
         {
            if(param1 == 0)
            {
               stopResubscribeTimer();
            }
            _resubscribeAttempts = param1;
         }
      }
      
      public function get timestamp() : Number
      {
         return _timestamp;
      }
      
      override mx_internal function setClientId(param1:String) : void
      {
         if(super.clientId != param1)
         {
            if(subscribed)
            {
               unsubscribe();
               _shouldBeSubscribed = true;
            }
            super.mx_internal::setClientId(param1);
            if(_shouldBeSubscribed)
            {
               subscribe(param1);
            }
         }
      }
      
      override public function channelDisconnectHandler(param1:ChannelEvent) : void
      {
         setSubscribed(false);
         super.channelDisconnectHandler(param1);
         if(_shouldBeSubscribed && !param1.rejected)
         {
            startResubscribeTimer();
         }
      }
      
      public function unsubscribe() : void
      {
         _shouldBeSubscribed = false;
         if(subscribed)
         {
            channelSet.removeEventListener(destination,messageHandler);
            if(Log.isInfo())
            {
               _log.info("\'{0}\' {1} unsubscribe.",id,_agentType);
            }
            internalSend(getUnsubscribeMessage());
         }
         else
         {
            stopResubscribeTimer();
         }
      }
      
      public function subscribe(param1:String = null) : void
      {
         var _loc2_:Boolean = false;
         if(session != null)
         {
            if(Log.isWarn() && param1 === null)
            {
               _log.warn("Client ID specified in subscribe is ignored when a session is assigned.");
            }
            param1 = session.clientId;
         }
         _loc2_ = param1 != null && super.clientId != param1 ? true : false;
         if(subscribed && _loc2_)
         {
            unsubscribe();
         }
         stopResubscribeTimer();
         _shouldBeSubscribed = true;
         if(_loc2_)
         {
            super.mx_internal::setClientId(param1);
         }
         if(Log.isInfo())
         {
            _log.info("\'{0}\' {1} subscribe.",id,_agentType);
         }
         _subscribeMsg = getSubscribeMessage();
         internalSend(_subscribeMsg);
      }
      
      public function get resubscribeInterval() : int
      {
         return _resubscribeInterval;
      }
      
      public function get resubscribeAttempts() : int
      {
         return _resubscribeAttempts;
      }
      
      override public function set subtopic(param1:String) : void
      {
         if(subtopic != param1)
         {
            if(subscribed)
            {
               unsubscribe();
               _shouldBeSubscribed = true;
            }
            super.subtopic = param1;
            if(_shouldBeSubscribed)
            {
               subscribe();
            }
         }
      }
      
      public function get messageType() : String
      {
         return _messageType;
      }
      
      protected function getSubscribeMessage() : CommandMessage
      {
         var _loc1_:CommandMessage = null;
         _loc1_ = new CommandMessage();
         _loc1_.operation = CommandMessage.SUBSCRIBE_OPERATION;
         _loc1_.clientId = clientId;
         _loc1_.messageRefType = messageType;
         _loc1_.destination = destination;
         if(_selector.length > 0)
         {
            _loc1_.headers[CommandMessage.SELECTOR_HEADER] = _selector;
         }
         return _loc1_;
      }
      
      [Bindable(event="propertyChange")]
      public function get subscribed() : Boolean
      {
         return _subscribed;
      }
      
      public function receive(param1:Number = 0) : void
      {
         var _loc2_:CommandMessage = null;
         _loc2_ = new CommandMessage();
         _loc2_.operation = CommandMessage.POLL_OPERATION;
         _loc2_.destination = destination;
         if(param1 == -1 || param1 > 0)
         {
            _loc2_.timestamp = param1;
         }
         else
         {
            _loc2_.timestamp = this.timestamp;
         }
         _loc2_.messageRefType = messageType;
         internalSend(_loc2_);
      }
      
      override public function channelFaultHandler(param1:ChannelFaultEvent) : void
      {
         if(!param1.channel.connected)
         {
            setSubscribed(false);
         }
         super.channelFaultHandler(param1);
         if(!param1.channel.connected && _shouldBeSubscribed)
         {
            startResubscribeTimer();
         }
      }
      
      public function get selector() : String
      {
         return _selector;
      }
      
      private function messageHandler(param1:MessageEvent) : void
      {
         var _loc2_:CommandMessage = null;
         if(param1.message is CommandMessage)
         {
            _loc2_ = param1.message as CommandMessage;
            switch(_loc2_.operation)
            {
               case CommandMessage.SESSION_INVALIDATE_OPERATION:
                  disconnect();
                  break;
               default:
                  if(Log.isWarn())
                  {
                     _log.warn("\'{0}\' received a CommandMessage \'{1}\' that could not be handled.",id,CommandMessage.getOperationAsString(_loc2_.operation));
                  }
            }
            return;
         }
         if(param1.message.timestamp > _timestamp)
         {
            _timestamp = param1.message.timestamp;
         }
         if(param1.message.headers[AbstractMessage.DESTINATION_CLIENT_ID_HEADER] == undefined || param1.message.headers[AbstractMessage.DESTINATION_CLIENT_ID_HEADER] == clientId)
         {
            dispatchEvent(MessageEvent.createEvent(MessageEvent.MESSAGE,param1.message));
         }
      }
      
      public function set selector(param1:String) : void
      {
         if(_selector !== param1)
         {
            _selector = param1;
            if(subscribed)
            {
               subscribe(clientId);
            }
         }
      }
   }
}

